 function [EL, IL, lambda] = bb_spectrum(TK, lambda, limits);
% function [EL, IL, lambda] = bb_spectrum(TK, lambda, limits);
% a lambda vector will be generated (400 points, up to 4*lmax)
% if not specified, provided TK is a single value.
%
% EL is the black body power spectral density (per metre), radiated over a
% hemisphere and assuming diffuse radiation.  So my EL is the qbl in Janna
% and EL = pi * IL where IL is the normal power density (per m wavelength)
% per steradian solid angle. 
% nb.  The pi multiplier arises when integrating over a hemisphere because
% of the cos(phi) multiplier in Lambert's law for a "diffuse emitter".
% This is why we don't just multiply by 2pi steradians.
%
% Specify limits = (eg) [3e-7 4e-6] to truncate to zer outside these
% wavelengths

h = 6.6256E-34;   % Planck's constant
k = 1.3805E-23;  % Boltzmann's constant
c = 2.998E8;  % speed of light


if nargin<2,
   if length(TK)~=1, 
       error('Must use a single T value if lambda not specified')
   end
   lmax = 2897.8E-6 / TK; 
   lambda = logspace(-0.9, 3, 150)*lmax;
end

hc_lkT = (h*c/k)./(lambda .* TK);

% IL is normal to surface, per steradian
IL = (2*h*c^2) ./ (lambda.^5 .* (exp(hc_lkT)-1)); 
if nargin>2,
    IL(lambda<limits(1)) = 0;
    IL(lambda>limits(2)) = 0;
end    

% EL is over a hemisphere, if diffuse.
EL = IL * pi;
