function [ints, fracs] = findel(clickx, varargin)
% function [ints, fracs] = findel(clickx, xvec1, xvec2, xvec3, xvec4): Finds element nearest to mouse x-position.
% FINDEL finds which point is closest to a given x-coordinate, which will typically
% be identified by calling ginput prior to findel.m.  clickx is a column vector; if more than
% one column, only the first column will be used (so can call as findel(ginput(n), xvec) etc).
% It is assumed that xvec is monotonic: findel is designed for use with plots of time or frequency-
% series data to identify regions for subsequent processing, calculating mean levels, etc.
% (If not monotonic, will interpolate between the first point >clickx & the previous point).
%
% xvec1 may be either a column vector (to identify points on a single line) or a matrix with each column 
% describing a line with different x-values. Extra input matrices can be specified as xvec2/3/4 if not
% same number of points as xvec1. 
% Returns [for click1, from: xvec1(cols1/2/3 etc) xvec2(ditto) etc
%         for click2,       ditto                   etc etc   ]
% as an integer matrix "ints" and a fractional matrix "fracs".
% (integer is closest point in range, fractional refers to position wrt 
% this range but may be < 1 or > length(var).
%
% If xvec is scalar, ints=1 and fracs=NaN.
% See also:  CLOSEST, REGIONS.
% RWM.

% Written by Roger Moss, February 1994.

nc = 0;
for nvar = 2:nargin,
  mn = size(varargin{nvar-1});
  if mn(1)==1,
     nc = nc + mn(1);
  else
     nc = nc + mn(2);
  end
end
clickx = clickx(:);
nclick = size(clickx, 1);
ints = zeros(nclick, nc);
fracs = zeros(nclick, nc);


ccol = 0;
for nvar = 1:nargin-1,
   xvec = varargin{nvar};
   [m n] = size(xvec);
   if m == 1,  %(rowwise data)
      xvec = xvec'; m = n; n=1;
   end
  
   if length(xvec)==0, error('Cannot use findel on a zero length variable');
   elseif length(xvec)==1,
      ints(:, ccol+1) = 1;
      fracs(:, ccol+1) = NaN;    
   else
      %---------------------------------------
      inn = [1:m]';
      for col = 1:n,
         % remove any NaN's which will confuse interpolation:
         nin = isnan(xvec(:,col));
         if any(nin), 
            nin = ~nin;
            xv = xvec(nin,col);
            inni = inn(nin);
         else
            xv = xvec(:,col);
            inni = inn;
         end
         
         f_col = table_rm([xv, inni], clickx, 1);
         fracs(:, ccol+col) = f_col;
         i_col = round(f_col);
         i_col = max(i_col, 1);
         i_col = min(i_col, m);
         ints(:, ccol+col) = i_col;
      end 	%col=1:n
   end
   ccol = ccol + n;
end	%nvar=1:nargin-1


