function [X, Y, W, Wsum, mstdW] = illumination3D_v3(s, n, ggabn, k, nx, max_reflections, p, target_frac)
% function [X, Y, W, Wsum, stdW] = illumination3D_v3(s, n, ggabn, k, nx, max_reflections, p, target_frac);
%
% s = half-width of square reflecting box
%
% height from target to bulbs = n*s
%
% ggabn is [gx, gy, alpha, rot, nb] for first bulb with coordinates [s*gx, s*gy, n*s];
% alpha is angle from vertically down;
% rot is angle anticlockwise from x-axis,about z-axis.  (both radians)
% nb = number of bulbs.  Alternatively if ggabn is nb*4, omit 5th value.
%
% Gap of height ks between box bottom edge and target (default k=0)
% target area 2s * 2s is covered by a uniform nx*nx grid (default nx = 21)
% Maximum of 2*max_reflections (max_reflections in both x and y-facing
% mirrors), default max_reflections = 7
%
% Layers of W are illumination by each light
% Wsum is sum over layers for all lights
% -------------------------------------------------------------------------
% This makes a 2D array of virtual image positions of a light source (one
% of them is the original light source) relative to the target point;
% for each one, finds distance from the target point,
% number of reflections, whether the beam escapes from bottom gap and beam
% angle relative to lamp axis.
% surf(X, Y, W) or surf(X, Y, Wsum)
%------------------------------------------------
% original values
% c = 21;v = round(0.22*c):c;
% [X, Y, W, Wsum, stdW] = illumination3D(0.26, 1.77/0.3, [0.46 0.5  0.13 -0.5*pi 4], 0.3, 2*c-1, 11);
% figure(1);surf(X,Y,Wsum);
% figure(2);plot(X(:,c), Wsum(:,c));
%
% p is polynomial coefficients from pyro3_p.mat
%
% _v3 with cos(theta) correction both in the lamp polynomial (pyro3_p.mat)
% and before summing rays!
% RWM

if nargin<8, target_frac = 1; end  % use 470/700 for mean power over a 470 mm wide collector
%-----------------------------------------------
%r = 0.894;  % reflection coefficient to match measured power
r = 0.88; % nominal

if iseven(max_reflections(1))
    max_reflections = max_reflections(1) + [0, -1];
else
    max_reflections = max_reflections(1) + [-1, 0];
end
nref_e = -max_reflections(1):2:max_reflections(1);
nref_o = -max_reflections(2):2:max_reflections(2);
nrefs = length(nref_e) + length(nref_o);

% +/- above simply indicates whether the virtual image lies to +x or -x of
% the source, I think??

fv = linspace(-1, 1, nx) * target_frac;  % non-dimensional points on target
[X,Y] = ndgrid(fv, fv);

ngg = size(ggabn);
% lamp = [0.17 0.19];
% lampcoor = [lamp(1)*[1, -1, -1, 1, 1]', lamp(2)*[1 1, -1, -1, 1]']/2;

if ngg(2)==5
    if ngg(1)~=1, error('nggabn must be a row vector if nb specified in column 5'); end
    nb = ggabn(1,5);
    ggab = ones(nb,1) * ggabn(1:4);
    for i = 2:nb  % dreate a new lamp in rotated position
       phi =  (2*pi/nb)*(i-1);
       M = [cos(phi) sin(phi); -sin(phi) cos(phi)];
       ggab(i, 1:2) = ggab(1, 1:2) * M;
       ggab(i, 4) = ggab(1,4) + phi;   
    end
else
    if ngg(2)~=4, error('nggabn must have 4 or 5 columns'); end
    ggab = ggabn;
    nb = ngg(1);
end
% make drawing coordinates for lamps
% lamps = [];
% for i = 1:nb
%        phi =  ggab(i,4);
%        M = [cos(phi) sin(phi); -sin(phi) cos(phi)];
%        lamps = [lamps; repmat(s*ggab(i, 1:2), [5 1]) + lampcoor * M; NaN NaN];
% end
% lamps = [lamps; s*[1 1; -1 1; -1 -1; 1 -1; 1 1]];

W = zeros([nx, nx, nb]);

for i = 1:nb    % loop through all the bulbs
    gx = ggab(i,1);
    gy = ggab(i,2);
    alpha = ggab(i,3);
    rot = ggab(i,4);
    % define bulb angle as a unit vector
    paraxial = reshape([sin(alpha)*[cos(rot) sin(rot)], cos(alpha)]', [1 1 3]);
    paraxial3 = zeros(nrefs, nrefs, 3);
    paraxial3(:) = repmat(paraxial, [nrefs nrefs 1]);
    
    for j = 1:nx          % loop through all target points (x)
        for jj = 1:nx     % loop through all target points (y)
            % now for each target point, from one bulb
            fx = X(j,jj);
            fy = Y(j,jj);
            xevens = 2*nref_e + fx - gx;  % displacements from g to f
            xodds = 2*nref_o - fx - gx;
            xside_shift = [xevens, xodds];
            
            yevens = 2*nref_e + fy - gy;
            yodds = 2*nref_o - fy - gy;
            yside_shift = [yevens, yodds];
            
            [XS, YS] = ndgrid(xside_shift, yside_shift);
            path_length = sqrt(XS.^2 + YS.^2 + n^2);
            
            [nrefx, nrefy] = ndgrid([nref_e, nref_o], [nref_e, nref_o]);
            fraction_reflected = r .^ (abs(nrefx)+abs(nrefy));
            
            
            % define matrix of unit vectors from virtual bulbs to target point:
            gfvectors = cat(3, XS, YS, ones(nrefs,nrefs)*n);
            cosbeta = dot(gfvectors, paraxial3, 3)./path_length;
            beta = acos(cosbeta);  % angle of each ray away from the floodlight axis
            
            costheta = n ./ path_length;  % angle of each ray from the target normal direction
            
            % check that light does not escape from bottom gap (height ks).
            % Look for intersection of light ray with horizontal surface at
            % height k; if within -1:+1, -1:+1 square, is OK.
            %
            % define rays in form r = a+lambda*b
            %  r = [fx, fy, 0]' + lambda * gfvectors
            % intersecting [0, 0, k]+....
            if k>0
                gfk = gfvectors * (k/n);
                XI = fx - gfk(:,:,1);
                YI = fy - gfk(:,:,2);
                usek = XI>=-1 & XI<=1 & YI>=-1 & YI<=1;
            else
                usek = true(nrefs,nrefs);
            end
            
            P = floodlight3(beta(usek), s * path_length(usek), p);  %  *1.015; % new bulb
                    
            Pref = P .* costheta(usek) .* fraction_reflected(usek);
            
            W(j, jj, i) = sum(Pref);
        end
    end
end

Wsum = sum(W, 3);

meanW = trapz2d(Wsum);
totW = meanW * (2*s)^2;
disp(['Mean power = ', num2str(meanW), ' W/m^2'])
disp(['Total power = ', num2str(totW), ' W'])
mstdW = [meanW std(Wsum(:))];

% n_edge = round((nx-1)/4); % eg 10 if nx = 41
% c_region = (1+n_edge):(nx-n_edge); % eg 11 to 31
% Wcent = Wsum(c_region, c_region);
% mstdW = [trapz2d(Wcent) std(Wcent(:))];
% displ('Mean target power = ', mstdW(1), ' W/m^2, Target std = ', mstdW(2), ' W/m^2')
% disp(' ')

%-----------------------------------------------
function P = floodlight3(angr, dist, p)
% function P = floodlight3(angr, dist);
% power (W/m2) from a 500W floodlight, angle (radians) angr off-axis,
% spherical shell distance dist from bulb.


if isempty(angr)
    disp('No rays!')
    P = 0;
else    
    P = max(polyval(p, angr*180/pi) ./ (dist.^2), 0);
end


