function [found1, from2, minmaxf] = textfind(matrix, string, exact);
% function [found, mask] = textfind(matrix, string, exact):  Equiv. to "find", for text matrices.
%
% Looks for occurrences in MATRIX of STRING: returns a list of rows "found" in which STRING
% is found and the mask equivalent of this such that found=find(mask);
% ("mask" allows use of & and | for further comparisons).
% If STRING is longer than MATRIX returns found=[].
% STRING '' returns found = [].
% 
% Can also be called as [found1, from2, minmaxf] = textfind(matrix1, matrix2)
% The use of a third output argument MINMAXF shows that a matrix interpretation is required.
% This searches through MATRIX1 for each row in MATRIX2 and combines every matching
% MATRIX1 row number into a column vector FOUND1. FROM2 is the same length as FOUND1
% and shows which row of MATRIX2 was in use when the corresponding row of FOUND1 was
% written, i.e for every row number n, matrix1(found1(n),1:m) == matrix2(from2(n),1:m))
% where m=size(matrix2,2);
% Output is sorted based on from2, i.e. from2 == sort(from2) always.
% MINMAXF(1) shows minimum number of "finds" from any row in MATRIX2, and MINMAXF(2)
% is maximum.    There is no "mask" argument in this case: if required it would have to
% be generated as mask = sparse(found1, from2, 1, length(matrix1), length(matrix2));
%
% MATRIX1 may also be a vector of form k+[1:n]', in which case it will be recognised as such
% and matrix2 elements which equal any matrix1 element (with a tolerance of 1E-14*max([1,
% largest element]) will be very rapidly identified.
%
% If "exact", pads string with blanks and compares all columns.
% See also FINDROW.
% RWM

% Written by Roger Moss, March 1994.
% NB: DATABASE assumes textfind output is sorted by from2 -
% if editing TEXTFIND, ENSURE THIS IS PRESERVED!

if iscellstr(matrix), matrix = char(matrix); end
if iscellstr(string), string = char(string); end
m = size(matrix);
n = size(string);
if nargin<3, exact = 0; end
if any([m n]==0),
   found1 = [];
   from2 = [];
   minmaxf = [0 0];
   return
end

if nargout<3,
 if n(1)>1,
  if m(1)>1,
   error('More than 1 row in string: call textfind(matrix, string) or [f1, f2, mmf]=textfind(IndexMatrix, RowsToBeFound)')
  else
   % swop string & matrix:
   temp = string; string = matrix; matrix = temp;
   temp = m; m = n; n = temp;
  end
 end
end
if n(2) > m(2),
  % error('String is longer than matrix words !')
  found1 = []; 
  if nargout<3,
   from2 = zeros(m(1), 1);
  else
   from2 = []; minmaxf = [0 0];
  end
  return
elseif n(2)<m(2),
 if exact,
    string = [string, blanks(n(1), m(2)-n(2))];
    n(2) = m(2);
 else
    matrix = matrix(:, 1:n(2));
    m(2) = n(2);
 end
end
%-------------------------------------------------------------------------------------
done=0;
if any([m(1) n(1)]==0),
 found1 = []; from2 = []; minmaxf = [0 0];
 done=1; ii = 0;
elseif m(2)==1 & matrix(m(1))-matrix(1)==m(1)-1,
 % trap case where matrix is [1:n]' column vector to make function faster:
 if all(matrix==matrix(1)+[0:m(1)-1]'),
  % note brackets around scalars: doubles speed and removes rounding errors:
  string = string+ (1-matrix(1));
  rstring = round(string);
  from2 = find(abs(string-rstring)<max([1; abs(matrix([1 m(1)],:))])*1e-14 & string>0 & string<m(1)+1);
  % find(mask) always returns a sorted vector of indices, so from2 is sorted
  found1 = rstring(from2);
  ii = length(found1);
  minmaxf = [ii==n(1), ii>0];
  done=1;
 end
end
%----------------------------------------------
if ~done,
 % 20 based on trial & error using rand(500,5) arrays: in practice,
 % which method is best will depend very much on how long mtx takes
 % to sort, which can be very variable depending on degree of repetition.
 if n(1)<=20,		
  matrix = matrix';
  found1 = []; from2 = []; nf1 = [];
  minmaxf = [1E12 0];
  for i = 1:n(1),
   if n(2)==1,
    found = find(matrix==string(i));
   else
    testm = string(i,:)'*ones(1,m(1));
    found = find(all(testm==matrix));
   end
   nf = length(found);
   found1 = [found1; found'];
   from2 = [from2; ones(nf,1)*i]; % for i=1:n, so from2 is sorted
   nf1 = [nf1; nf];
  end
  minmaxf = [min([minmaxf(1); nf1]) max([minmaxf(2); nf1])];
  ii = sum(nf1);
  minmaxf(1)=minmaxf(1)*(ii>0);
 %---------------------------------
 % else use more complex algorithm to save time:
 % (up to 20* faster for large arrays)
 else
  mtx = double([[matrix zeros(m(1),1)]; [string ones(n(1),1)]]);
%  mtx(1) = mtx(1)+0;		% convert to numerical so can use diff later.
  [mtx, I] = textsort(mtx);
  ivec = logical(mtx(:,n(2)+1)+0);  % + 0 for make double array
  rvec = [1:m(1), 1:n(1)]';
  rvec = rvec(I);		% shows which row elements of mtx came from. 

  % Now split into groups of equal words and select groups that
  % start with a matrix word and finish with a string word. 

  if n(2)==1,
   dn = find(diff(mtx(:, 1))==0);
  else
  dn = find(all(diff(mtx(:, 1:n(2)))'==0))';
  end
  % So dn lists row numbers in mtx that are the same as the previous row.

  if any(dn),
   ddn = [0; find(diff(dn)>1); length(dn)]+1;
   % So mtx(dn(ddn(a)),:) is first row in group number a, and
   % mtx(dn(ddn(a+1)-1),:) is last row in group a.
   first = dn(ddn(1:length(ddn)-1));
   last = dn(ddn(2:length(ddn))-1)+1;
   replengths = last + 1 - first;
   wanted = find(mtx(first, n(2)+1)==0 & mtx(last, n(2)+1)==1); 
   maxlen = sum(replengths(wanted))-length(wanted);  % Just a guess, may be more or less
   ii = 0;  ii2 = 0;
   found1 = zeros(maxlen,1); from2 = found1; mx = 0; mn = Inf;

   for i = 1:length(wanted),
    nw = first(wanted(i)):last(wanted(i));  
    inw = ivec(nw);
    nb = sum(inw); na = length(inw)-nb; newr = na*nb;
    if ii+newr>maxlen,
     found1 = [found1; zeros(2000+newr,1)];
     from2 = [from2; zeros(2000+newr,1)];
     maxlen = maxlen+2000+newr;
    end
    rnw = rvec(nw);
    rows1 = rnw(~inw)*ones(1, nb);
    rows2 = ones(na,1)*rnw(inw)';
    found1(ii+[1:newr])=rows1(:);
    from2(ii+[1:newr])=rows2(:);
    ii = ii+newr;
    ii2 = ii2+nb;
    mx = max([mx, na]);
    mn = min([mn, na]);
   end
   [from2, I] = sort(from2(1:ii));
   found1 = found1(I);
   if ~ii | ii2<n(1), mn = 0; end
   minmaxf = [mn mx];
  else	% no dn repeats:
   found1 = [];
   from2 = [];
   minmaxf = [0 0];
   ii = 0;
  end
 end
end

if nargout==2,
 from2 = zeros(m(1),1);
 from2(found1) = ones(ii,1);
end

